C
C     Motion for force acting on plate in spherical geometry. Constant coupling
C
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (PI=3.14159265358979324D0,
     +           CONV = PI / 1.8D2)
      dimension vlat(100),vlon(100)
C
C     Specify plume location
C     rd is the radius of a disk with same volume as the plumehead normalized to Earth radius
C     rmax is the maximum radius to where plume-induced flow is considered
C     hc is the thickness of the channel with Poiseuille flow induced by the plume
C     hs is the effective layer thickness of the shearing due to plate motions
C
C      flat=-46.9
C      flon=37.8
C      flat=-40.
C      flon=41.5
C      flat=-44.676
C      flon=36.4971
C      flat=-41.4088
C      flon=25.6180
      open (3,file='madagascar_recon_87.d')
C      open (3,file='madagascar_test.d')
      read (3,*) flon,flat
C      flat=-40.4166
C      flon=27.0507
      close (3)
      rp=500.
C      hc=120.
C      hs=120.
C      hc=156.21
      hc=203.37
      hs=203.37
      rd=2*rp*sqrt(rp/3/hc)/6371.
      rmax=2400./6371.
C
C     Point in cartesian coordinates
C
      xplu=cos(flat*conv)*cos(flon*conv)
      yplu=cos(flat*conv)*sin(flon*conv)
      zplu=sin(flat*conv)
C
C     Torques for unit rotation around three axes and for flow radially away from plume
C
      txx=0.
      txy=0.
      txz=0.
      tyx=0.
      tyy=0.
      tyz=0.
      tzx=0.
      tzy=0.
      tzz=0.
      t0x=0.
      t0y=0.
      t0z=0.
      ivel=0
      do i=1,2000000
        read (5,*,end=99) plat,plon
        if ((int((plat+1000.)*10.)/100.eq.(plat+1000)/10.).and.
     2     (int((plon+1000.)*10.)/100.eq.(plon+1000)/10.)) then
          ivel=ivel+1
          vlat(ivel)=plat
          vlon(ivel)=plon
        endif
        x=cos(plat*conv)*cos(plon*conv)
        y=cos(plat*conv)*sin(plon*conv)
        z=sin(plat*conv)
        a=sqrt(1-z*z)
C
C     Torque as cross product r x F
C
C idir=1
        vx=0.
        vy=-z
        vz=y
        txx=txx+(y*vz-z*vy)*a
        txy=txy+(z*vx-x*vz)*a
        txz=txz+(x*vy-y*vx)*a
C idir=2
        vx=z
        vy=0
        vz=-x
        tyx=tyx+(y*vz-z*vy)*a
        tyy=tyy+(z*vx-x*vz)*a
        tyz=tyz+(x*vy-y*vx)*a
C idir=3
        vx=-y
        vy=x
        vz=0
        tzx=tzx+(y*vz-z*vy)*a
        tzy=tzy+(z*vx-x*vz)*a
        tzz=tzz+(x*vy-y*vx)*a
C Motion radially away from plume
C First cross product: Unit vector counterclockwise around plume         
        xtan=yplu*z-zplu*y
        ytan=zplu*x-xplu*z
        ztan=xplu*y-yplu*x
        rtan=sqrt(xtan*xtan+ytan*ytan+ztan*ztan)
C Second cross product: Unit vector radially away from the plume 
C Distinction between two cases in order to exclude pathological case where x=xplu etc
C In this case (distance < 0.05 deg) set radial vector to zero
        dist=sqrt((x-xplu)*(x-xplu)+(y-yplu)*(y-yplu)
     2            +(z-zplu)*(z-zplu))
        if (dist.gt.pi/3600.) then
          xrad=(ytan*z-ztan*y)/rtan
          yrad=(ztan*x-xtan*z)/rtan
          zrad=(xtan*y-ytan*x)/rtan
        else
          xrad=0.
          yrad=0.
          zrad=0.
        endif
C Multiply radial vector with radial function
        if ((dist.lt.rmax).and.(dist.gt.pi/3600.)) then
C          xrad=xrad*rd/2.
C          yrad=yrad*rd/2.
C          zrad=zrad*rd/2.
C        else
          xrad=xrad*rd*rd/2./dist
          yrad=yrad*rd*rd/2./dist
          zrad=zrad*rd*rd/2./dist
        else
          xrad=0.
          yrad=0.
          zrad=0.
        endif
        t0x=t0x+(y*zrad-z*yrad)*a
        t0y=t0y+(z*xrad-x*zrad)*a
        t0z=t0z+(x*yrad-y*xrad)*a
      enddo
 99   continue
C      print *,txx,tyx,tzx
C      print *,txy,tyy,tzy
C      print *,txz,tyz,tzz
      det0=txx*(tyy*tzz-tyz*tzy)+tyx*(txz*tzy-txy*tzz)
     2    +tzx*(txy*tyz-txz*tyy)
      det1=t0x*(tyy*tzz-tyz*tzy)+tyx*(t0z*tzy-t0y*tzz)
     2    +tzx*(t0y*tyz-t0z*tyy)
      det2=txx*(t0y*tzz-t0z*tzy)+t0x*(txz*tzy-txy*tzz)
     2    +tzx*(txy*t0z-txz*t0y)
      det3=txx*(tyy*t0z-tyz*t0y)+tyx*(txz*t0y-txy*t0z)
     2    +t0x*(txy*tyz-txz*tyy)
      open (8,file='omega.d')
      write (8,*) det1/det0*hs/hc,det2/det0*hs/hc,det3/det0*hs/hc
      close (8)
      open (3,file='omega_sub.d')
      read (3,*) omx0,omy0,omz0
      close (3)
      omx=det1/det0*hs/hc-omx0
      omy=det2/det0*hs/hc-omy0
      omz=det3/det0*hs/hc-omz0
C      omx=det1/det0*hs/hc+7.7096412750233996E-003
C      omy=det2/det0*hs/hc+3.8002585038339090E-003
C      omz=det3/det0*hs/hc+3.6489982750573581E-003
      call car2sph(omx,omy,omz,omlat,omlon,om0)
      call car2sph(t0x,t0y,t0z,tlat,tlon,t0)
C      print *,tlat,tlon,t0
C      print *,omx,omy,omz,omlat,omlon,6.*om0/conv
C      stop
C
C     compute velocities on regular (10 degree) grid
C
      nvel=ivel
      do ivel=1,nvel
        omx=cos(omlat*conv)*cos((omlon-vlon(ivel))*conv)*om0/conv*6.
        omy=cos(omlat*conv)*sin((omlon-vlon(ivel))*conv)*om0/conv*6.
        omz=sin(omlat*conv)*om0/conv*6.
        ptx=cos(vlat(ivel)*conv)
        ptz=sin(vlat(ivel)*conv)
        vx=omy*ptz
        vy=omz*ptx-omx*ptz
        vz=-omy*ptx
        ve=vy
        vn=vz*sqrt((ptx*ptx+ptz*ptz)/ptx/ptx)
        sinlat=sin((90.-vlat(ivel)-vn)*pi/180.)
        write (6,10)
   10   format ('>')
        print *,vlat(ivel),vlon(ivel)
        print *,vlat(ivel)+vn,vlon(ivel)+ve/sinlat
        arl=sqrt(ve*ve+vn*vn)
        hl=1.2
        hw=0.6
        hbe=ve*(arl-hl)/arl
        hbn=vn*(arl-hl)/arl
        he2=hbe+vn/arl*hw
        he3=hbe-vn/arl*hw
        hn2=hbn-ve/arl*hw
        hn3=hbn+ve/arl*hw
        write (6,10)
        print *,vlat(ivel)+vn,vlon(ivel)+ve/sinlat
        print *,vlat(ivel)+hn2,vlon(ivel)+he2/sinlat
        write (6,10)
        print *,vlat(ivel)+vn,vlon(ivel)+ve/sinlat
        print *,vlat(ivel)+hn3,vlon(ivel)+he3/sinlat
      enddo

      end

      subroutine car2sph(x,y,z,rlat,rlon,rad)
      implicit real*8 (a-h,o-z)

      PARAMETER (PI=3.14159265358979324D0,
     +           CONV = PI / 1.8D2)
      rad=sqrt(x*x+y*y+z*z)
      if (rad.eq.0) then
        rlon=0.
        rlat=0.
      else if ((y.ne.0).or.(x.ne.0)) then
        rlat=atan(z/sqrt(x*x+y*y))/conv
        if (x.gt.0) then
          rlon=atan(y/x)/conv
        else if ((x.lt.0).and.(y.ge.0)) then
          rlon=atan(y/x)/conv+180.
        else if ((x.lt.0).and.(y.lt.0)) then
          rlon=atan(y/x)/conv-180.
        else if ((x.eq.0).and.(y.gt.0)) then
          rlon=90.
        else if ((x.eq.0).and.(y.lt.0)) then
          rlon=-90.
        endif
      else if (z.gt.0) then
        rlat=90.
        rlon=0.
      else
        rlat=-90.
        rlon=0.
      endif
      end
